//
//	RemoteJdbcJdbcServer.java
//
//	version 1.0  98/12/12
//	author  Mitsumasa IWATSUKI
//

package org.iwatsuki.rjj;

import java.io.*;
import java.math.BigDecimal;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.sql.*;

public class RemoteJdbcJdbcServer extends UnicastRemoteObject implements RemoteDriver {
	public RemoteJdbcJdbcServer() throws RemoteException {
	}

	public RemoteConnection connect(String url, java.util.Properties info) throws RemoteException, SQLException {
		return new RemoteConnectionImpl(DriverManager.getDriver(url).connect(url, info));
	}

	public boolean acceptsURL(String url) throws RemoteException, SQLException {
		java.util.Enumeration drivers = DriverManager.getDrivers();
		while (drivers.hasMoreElements()) {
			Driver driver = (Driver)drivers.nextElement();
			if (driver.acceptsURL(url))
				return true;
		}
		return false;
	}

	public DriverPropertyInfo[] getPropertyInfo(String url, java.util.Properties info) throws RemoteException, SQLException {
		return DriverManager.getDriver(url).getPropertyInfo(url, info);
	}

	public static void main(String[] args) {
		try {
			String rmiURL = "";
			String security = "";
			for (int i = 0; i < args.length; i++) {
				String arg = args[i];
				if (arg.startsWith("-rjj:")) {
					rmiURL = arg.substring(5).trim();
				} else if (arg.startsWith("-security=")) {
					security = arg.substring(10).trim();
				} else {
					String driver = arg;
					Class.forName(driver);
					System.out.println(driver + " loaded");
				}
			}

			String name = "";
			if (rmiURL.startsWith("//")) {
				int index = rmiURL.indexOf('/', 2);
				if (index == -1)
					rmiURL += '/';
				else
					name = rmiURL.substring(++index).trim();
			} else if (rmiURL.startsWith("/")) {
				name = rmiURL.substring(1).trim();
			} else {
				name = rmiURL;
			}

			if (name.equals("")) {
				name = "RemoteDriver";
				rmiURL += name;
			}

			if (!security.equals("")) {
				System.setSecurityManager((SecurityManager)Class.forName(security).newInstance());
				System.out.println(security + " set");
			}
			Naming.rebind(rmiURL, new RemoteJdbcJdbcServer());
			System.out.println("RemoteJdbcJdbcServer bound in the registry to the name " + name);
		} catch (Exception e) {
			System.out.println("RemoteJdbcJdbcServer error: " + e.getMessage());
		}
	}
}

class RemoteConnectionImpl extends UnicastRemoteObject implements RemoteConnection {
	private Connection con;

	public RemoteConnectionImpl(Connection con) throws RemoteException {
		this.con = con;
	}

	//----------------------------------------------------------------------

	public RemoteStatement createStatement() throws RemoteException, SQLException {
		return new RemoteStatementImpl(con.createStatement());
	}

	public RemotePreparedStatement prepareStatement(String sql) throws RemoteException, SQLException {
		return new RemotePreparedStatementImpl(con.prepareStatement(sql));
	}

	public RemoteCallableStatement prepareCall(String sql) throws RemoteException, SQLException {
		return new RemoteCallableStatementImpl(con.prepareCall(sql));
	}

	public String nativeSQL(String sql) throws RemoteException, SQLException {
		return con.nativeSQL(sql);
	}

	public void setAutoCommit(boolean autoCommit) throws RemoteException, SQLException {
		con.setAutoCommit(autoCommit);
	}

	public boolean getAutoCommit() throws RemoteException, SQLException {
		return con.getAutoCommit();
	}

	public void commit() throws RemoteException, SQLException {
		con.commit();
	}

	public void rollback() throws RemoteException, SQLException {
		con.rollback();
	}

	public void close() throws RemoteException, SQLException {
		con.close();
	}

	public boolean isClosed() throws RemoteException, SQLException {
		return con.isClosed();
	}

	//======================================================================
	// Advanced features:

	public RemoteDatabaseMetaData getMetaData() throws RemoteException, SQLException {
		return new RemoteDatabaseMetaDataImpl(con.getMetaData());
	}

	public void setReadOnly(boolean readOnly) throws RemoteException, SQLException {
		con.setReadOnly(readOnly);
	}

	public boolean isReadOnly() throws RemoteException, SQLException {
		return con.isReadOnly();
	}

	public void setCatalog(String catalog) throws RemoteException, SQLException {
		con.setCatalog(catalog);
	}

	public String getCatalog() throws RemoteException, SQLException {
		return con.getCatalog();
	}

	public void setTransactionIsolation(int level) throws RemoteException, SQLException {
		con.setTransactionIsolation(level);
	}

	public int getTransactionIsolation() throws RemoteException, SQLException {
		return con.getTransactionIsolation();
	}

	public SQLWarning getWarnings() throws RemoteException, SQLException {
		return con.getWarnings();
	}

	public void clearWarnings() throws RemoteException, SQLException {
		con.clearWarnings();
	}
}

class RemoteStatementImpl extends UnicastRemoteObject implements RemoteStatement {
	private Statement stmt;

	public RemoteStatementImpl(Statement stmt) throws RemoteException {
		this.stmt = stmt;
	}

	//----------------------------------------------------------------------

	public RemoteResultSet executeQuery(String sql) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(stmt.executeQuery(sql));
	}

	public int executeUpdate(String sql) throws RemoteException, SQLException {
		return stmt.executeUpdate(sql);
	}

	public void close() throws RemoteException, SQLException {
		stmt.close();
	}

	//----------------------------------------------------------------------

	public int getMaxFieldSize() throws RemoteException, SQLException {
		return stmt.getMaxFieldSize();
	}

	public void setMaxFieldSize(int max) throws RemoteException, SQLException {
		stmt.setMaxFieldSize(max);
	}

	public int getMaxRows() throws RemoteException, SQLException {
		return stmt.getMaxRows();
	}

	public void setMaxRows(int max) throws RemoteException, SQLException {
		stmt.setMaxRows(max);
	}

	public void setEscapeProcessing(boolean enable) throws RemoteException, SQLException {
		stmt.setEscapeProcessing(enable);
	}

	public int getQueryTimeout() throws RemoteException, SQLException {
		return stmt.getQueryTimeout();
	}

	public void setQueryTimeout(int seconds) throws RemoteException, SQLException {
		stmt.setQueryTimeout(seconds);
	}

	public void cancel() throws RemoteException, SQLException {
		stmt.cancel();
	}

	public SQLWarning getWarnings() throws RemoteException, SQLException {
		return stmt.getWarnings();
	}

	public void clearWarnings() throws RemoteException, SQLException {
		stmt.clearWarnings();
	}

	public void setCursorName(String name) throws RemoteException, SQLException {
		stmt.setCursorName(name);
	}

	//----------------------- Multiple Results --------------------------

	public boolean execute(String sql) throws RemoteException, SQLException {
		return stmt.execute(sql);
	}

	public RemoteResultSet getResultSet() throws RemoteException, SQLException {
		return new RemoteResultSetImpl(stmt.getResultSet());
	}

	public int getUpdateCount() throws RemoteException, SQLException {
		return stmt.getUpdateCount();
	}

	public boolean getMoreResults() throws RemoteException, SQLException {
		return stmt.getMoreResults();
	}
}

class RemotePreparedStatementImpl extends RemoteStatementImpl implements RemotePreparedStatement {
	private PreparedStatement pstmt;

	public RemotePreparedStatementImpl(PreparedStatement pstmt) throws RemoteException {
		super(pstmt);
		this.pstmt = pstmt;
	}

	//----------------------------------------------------------------------

	public RemoteResultSet executeQuery() throws RemoteException, SQLException {
		return new RemoteResultSetImpl(pstmt.executeQuery());
	}

	public int executeUpdate() throws RemoteException, SQLException {
		return pstmt.executeUpdate();
	}

	public void setNull(int parameterIndex, int sqlType) throws RemoteException, SQLException {
		pstmt.setNull(parameterIndex, sqlType);
	}

	public void setBoolean(int parameterIndex, boolean x) throws RemoteException, SQLException {
		pstmt.setBoolean(parameterIndex, x);
	}

	public void setByte(int parameterIndex, byte x) throws RemoteException, SQLException {
		pstmt.setByte(parameterIndex, x);
	}

	public void setShort(int parameterIndex, short x) throws RemoteException, SQLException {
		pstmt.setShort(parameterIndex, x);
	}

	public void setInt(int parameterIndex, int x) throws RemoteException, SQLException {
		pstmt.setInt(parameterIndex, x);
	}

	public void setLong(int parameterIndex, long x) throws RemoteException, SQLException {
		pstmt.setLong(parameterIndex, x);
	}

	public void setFloat(int parameterIndex, float x) throws RemoteException, SQLException {
		pstmt.setFloat(parameterIndex, x);
	}

	public void setDouble(int parameterIndex, double x) throws RemoteException, SQLException {
		pstmt.setDouble(parameterIndex, x);
	}

	public void setBigDecimal(int parameterIndex, BigDecimal x) throws RemoteException, SQLException {
		pstmt.setBigDecimal(parameterIndex, x);
	}

	public void setString(int parameterIndex, String x) throws RemoteException, SQLException {
		pstmt.setString(parameterIndex, x);
	}

	public void setBytes(int parameterIndex, byte x[]) throws RemoteException, SQLException {
		pstmt.setBytes(parameterIndex, x);
	}

	public void setDate(int parameterIndex, java.sql.Date x) throws RemoteException, SQLException {
		pstmt.setDate(parameterIndex, x);
	}

	public void setTime(int parameterIndex, java.sql.Time x) throws RemoteException, SQLException {
		pstmt.setTime(parameterIndex, x);
	}

	public void setTimestamp(int parameterIndex, java.sql.Timestamp x) throws RemoteException, SQLException {
		pstmt.setTimestamp(parameterIndex, x);
	}

	public void setAsciiStream(int parameterIndex, byte x[], int length) throws RemoteException, SQLException {
		pstmt.setAsciiStream(parameterIndex, new ByteArrayInputStream(x), length);
	}

	public void setUnicodeStream(int parameterIndex, byte x[], int length) throws RemoteException, SQLException {
		pstmt.setUnicodeStream(parameterIndex, new ByteArrayInputStream(x), length);
	}

	public void setBinaryStream(int parameterIndex, byte x[], int length) throws RemoteException, SQLException {
		pstmt.setBinaryStream(parameterIndex, new ByteArrayInputStream(x), length);
	}

	public void clearParameters() throws RemoteException, SQLException {
		pstmt.clearParameters();
	}

	//----------------------------------------------------------------------
	// Advanced features:

	public void setObject(int parameterIndex, Object x, int targetSqlType, int scale) throws RemoteException, SQLException {
		pstmt.setObject(parameterIndex, x, targetSqlType, scale);
	}

	public void setObject(int parameterIndex, Object x, int targetSqlType) throws RemoteException, SQLException {
		pstmt.setObject(parameterIndex, x, targetSqlType);
	}

	public void setObject(int parameterIndex, Object x) throws RemoteException, SQLException {
		pstmt.setObject(parameterIndex, x);
	}

	public boolean execute() throws RemoteException, SQLException {
		return pstmt.execute();
	}
}

class RemoteCallableStatementImpl extends RemotePreparedStatementImpl implements RemoteCallableStatement {
	private CallableStatement cstmt;

	public RemoteCallableStatementImpl(CallableStatement cstmt) throws RemoteException {
		super(cstmt);
		this.cstmt = cstmt;
	}

	//----------------------------------------------------------------------

	public void registerOutParameter(int parameterIndex, int sqlType) throws RemoteException, SQLException {
		cstmt.registerOutParameter(parameterIndex, sqlType);
	}

	public void registerOutParameter(int parameterIndex, int sqlType, int scale) throws RemoteException, SQLException {
		cstmt.registerOutParameter(parameterIndex, sqlType, scale);
	}

	public boolean wasNull() throws RemoteException, SQLException {
		return cstmt.wasNull();
	}

	public String getString(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getString(parameterIndex);
	}

	public boolean getBoolean(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getBoolean(parameterIndex);
	}

	public byte getByte(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getByte(parameterIndex);
	}

	public short getShort(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getShort(parameterIndex);
	}

	public int getInt(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getInt(parameterIndex);
	}

	public long getLong(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getLong(parameterIndex);
	}

	public float getFloat(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getFloat(parameterIndex);
	}

	public double getDouble(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getDouble(parameterIndex);
	}

	public BigDecimal getBigDecimal(int parameterIndex, int scale) throws RemoteException, SQLException {
		return cstmt.getBigDecimal(parameterIndex, scale);
	}

	public byte[] getBytes(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getBytes(parameterIndex);
	}

	public java.sql.Date getDate(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getDate(parameterIndex);
	}

	public java.sql.Time getTime(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getTime(parameterIndex);
	}

	public java.sql.Timestamp getTimestamp(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getTimestamp(parameterIndex);
	}

	//----------------------------------------------------------------------
	// Advanced features:

	public Object getObject(int parameterIndex) throws RemoteException, SQLException {
		return cstmt.getObject(parameterIndex);
	}
}

class RemoteResultSetImpl extends UnicastRemoteObject implements RemoteResultSet {
	private ResultSet rs;

	public RemoteResultSetImpl(ResultSet rs) throws RemoteException {
		this.rs = rs;
	}

	//----------------------------------------------------------------------

	public boolean next() throws RemoteException, SQLException {
		return rs.next();
	}

	public void close() throws RemoteException, SQLException {
		rs.close();
	}

	public boolean wasNull() throws RemoteException, SQLException {
		return rs.wasNull();
	}

	//======================================================================
	// Methods for accessing results by column index
	//======================================================================

	public String getString(int columnIndex) throws RemoteException, SQLException {
		return rs.getString(columnIndex);
	}

	public boolean getBoolean(int columnIndex) throws RemoteException, SQLException {
		return rs.getBoolean(columnIndex);
	}

	public byte getByte(int columnIndex) throws RemoteException, SQLException {
		return rs.getByte(columnIndex);
	}

	public short getShort(int columnIndex) throws RemoteException, SQLException {
		return rs.getShort(columnIndex);
	}

	public int getInt(int columnIndex) throws RemoteException, SQLException {
		return rs.getInt(columnIndex);
	}

	public long getLong(int columnIndex) throws RemoteException, SQLException {
		return rs.getLong(columnIndex);
	}

	public float getFloat(int columnIndex) throws RemoteException, SQLException {
		return rs.getFloat(columnIndex);
	}

	public double getDouble(int columnIndex) throws RemoteException, SQLException {
		return rs.getDouble(columnIndex);
	}

	public BigDecimal getBigDecimal(int columnIndex, int scale) throws RemoteException, SQLException {
		return rs.getBigDecimal(columnIndex, scale);
	}

	public byte[] getBytes(int columnIndex) throws RemoteException, SQLException {
		return rs.getBytes(columnIndex);
	}

	public java.sql.Date getDate(int columnIndex) throws RemoteException, SQLException {
		return rs.getDate(columnIndex);
	}

	public java.sql.Time getTime(int columnIndex) throws RemoteException, SQLException {
		return rs.getTime(columnIndex);
	}

	public java.sql.Timestamp getTimestamp(int columnIndex) throws RemoteException, SQLException {
		return rs.getTimestamp(columnIndex);
	}

	public RemoteInputStream getAsciiStream(int columnIndex) throws RemoteException, SQLException {
		return new RemoteInputStreamImpl(rs.getAsciiStream(columnIndex));
	}

	public RemoteInputStream getUnicodeStream(int columnIndex) throws RemoteException, SQLException {
		return new RemoteInputStreamImpl(rs.getUnicodeStream(columnIndex));
	}

	public RemoteInputStream getBinaryStream(int columnIndex) throws RemoteException, SQLException {
		return new RemoteInputStreamImpl(rs.getBinaryStream(columnIndex));
	}

	//======================================================================
	// Methods for accessing results by column name
	//======================================================================

	public String getString(String columnName) throws RemoteException, SQLException {
		return rs.getString(columnName);
	}

	public boolean getBoolean(String columnName) throws RemoteException, SQLException {
		return rs.getBoolean(columnName);
	}

	public byte getByte(String columnName) throws RemoteException, SQLException {
		return rs.getByte(columnName);
	}

	public short getShort(String columnName) throws RemoteException, SQLException {
		return rs.getShort(columnName);
	}

	public int getInt(String columnName) throws RemoteException, SQLException {
		return rs.getInt(columnName);
	}

	public long getLong(String columnName) throws RemoteException, SQLException {
		return rs.getLong(columnName);
	}

	public float getFloat(String columnName) throws RemoteException, SQLException {
		return rs.getFloat(columnName);
	}

	public double getDouble(String columnName) throws RemoteException, SQLException {
		return rs.getDouble(columnName);
	}

	public BigDecimal getBigDecimal(String columnName, int scale) throws RemoteException, SQLException {
		return rs.getBigDecimal(columnName, scale);
	}

	public byte[] getBytes(String columnName) throws RemoteException, SQLException {
		return rs.getBytes(columnName);
	}

	public java.sql.Date getDate(String columnName) throws RemoteException, SQLException {
		return rs.getDate(columnName);
	}

	public java.sql.Time getTime(String columnName) throws RemoteException, SQLException {
		return rs.getTime(columnName);
	}

	public java.sql.Timestamp getTimestamp(String columnName) throws RemoteException, SQLException {
		return rs.getTimestamp(columnName);
	}

	public RemoteInputStream getAsciiStream(String columnName) throws RemoteException, SQLException {
		return new RemoteInputStreamImpl(rs.getAsciiStream(columnName));
	}

	public RemoteInputStream getUnicodeStream(String columnName) throws RemoteException, SQLException {
		return new RemoteInputStreamImpl(rs.getUnicodeStream(columnName));
	}

	public RemoteInputStream getBinaryStream(String columnName) throws RemoteException, SQLException {
		return new RemoteInputStreamImpl(rs.getBinaryStream(columnName));
	}

	//=====================================================================
	// Advanced features:
	//=====================================================================

	public SQLWarning getWarnings() throws RemoteException, SQLException {
		return rs.getWarnings();
	}

	public void clearWarnings() throws RemoteException, SQLException {
		rs.clearWarnings();
	}

	public String getCursorName() throws RemoteException, SQLException {
		return rs.getCursorName();
	}

	public RemoteResultSetMetaData getMetaData() throws RemoteException, SQLException {
		return new RemoteResultSetMetaDataImpl(rs.getMetaData());
	}

	public Object getObject(int columnIndex) throws RemoteException, SQLException {
		return rs.getObject(columnIndex);
	}

	public Object getObject(String columnName) throws RemoteException, SQLException {
		return rs.getObject(columnName);
	}

	//----------------------------------------------------------------

	public int findColumn(String columnName) throws RemoteException, SQLException {
		return rs.findColumn(columnName);
	}
}

class RemoteDatabaseMetaDataImpl extends UnicastRemoteObject implements RemoteDatabaseMetaData {
	private DatabaseMetaData dm;

	public RemoteDatabaseMetaDataImpl(DatabaseMetaData dm) throws RemoteException {
		this.dm = dm;
	}

	//----------------------------------------------------------------------
	// First, a variety of minor information about the target database.

	public boolean allProceduresAreCallable() throws RemoteException, SQLException {
		return dm.allProceduresAreCallable();
	}

	public boolean allTablesAreSelectable() throws RemoteException, SQLException {
		return dm.allTablesAreSelectable();
	}

	public String getURL() throws RemoteException, SQLException {
		return dm.getURL();
	}

	public String getUserName() throws RemoteException, SQLException {
		return dm.getUserName();
	}

	public boolean isReadOnly() throws RemoteException, SQLException {
		return dm.isReadOnly();
	}

	public boolean nullsAreSortedHigh() throws RemoteException, SQLException {
		return dm.nullsAreSortedHigh();
	}

	public boolean nullsAreSortedLow() throws RemoteException, SQLException {
		return dm.nullsAreSortedLow();
	}

	public boolean nullsAreSortedAtStart() throws RemoteException, SQLException {
		return dm.nullsAreSortedAtStart();
	}

	public boolean nullsAreSortedAtEnd() throws RemoteException, SQLException {
		return dm.nullsAreSortedAtEnd();
	}

	public String getDatabaseProductName() throws RemoteException, SQLException {
		return dm.getDatabaseProductName();
	}

	public String getDatabaseProductVersion() throws RemoteException, SQLException {
		return dm.getDatabaseProductVersion();
	}

	public String getDriverName() throws RemoteException, SQLException {
		return dm.getDriverName();
	}

	public String getDriverVersion() throws RemoteException, SQLException {
		return dm.getDriverVersion();
	}

	public int getDriverMajorVersion() throws RemoteException {
		return dm.getDriverMajorVersion();
	}

	public int getDriverMinorVersion() throws RemoteException {
		return dm.getDriverMinorVersion();
	}

	public boolean usesLocalFiles() throws RemoteException, SQLException {
		return dm.usesLocalFiles();
	}

	public boolean usesLocalFilePerTable() throws RemoteException, SQLException {
		return dm.usesLocalFilePerTable();
	}

	public boolean supportsMixedCaseIdentifiers() throws RemoteException, SQLException {
		return dm.supportsMixedCaseIdentifiers();
	}

	public boolean storesUpperCaseIdentifiers() throws RemoteException, SQLException {
		return dm.storesUpperCaseIdentifiers();
	}

	public boolean storesLowerCaseIdentifiers() throws RemoteException, SQLException {
		return dm.storesLowerCaseIdentifiers();
	}

	public boolean storesMixedCaseIdentifiers() throws RemoteException, SQLException {
		return dm.storesMixedCaseIdentifiers();
	}

	public boolean supportsMixedCaseQuotedIdentifiers() throws RemoteException, SQLException {
		return dm.supportsMixedCaseQuotedIdentifiers();
	}

	public boolean storesUpperCaseQuotedIdentifiers() throws RemoteException, SQLException {
		return dm.storesUpperCaseQuotedIdentifiers();
	}

	public boolean storesLowerCaseQuotedIdentifiers() throws RemoteException, SQLException {
		return dm.storesLowerCaseQuotedIdentifiers();
	}

	public boolean storesMixedCaseQuotedIdentifiers() throws RemoteException, SQLException {
		return dm.storesMixedCaseQuotedIdentifiers();
	}

	public String getIdentifierQuoteString() throws RemoteException, SQLException {
		return dm.getIdentifierQuoteString();
	}

	public String getSQLKeywords() throws RemoteException, SQLException {
		return dm.getSQLKeywords();
	}

	public String getNumericFunctions() throws RemoteException, SQLException {
		return dm.getNumericFunctions();
	}

	public String getStringFunctions() throws RemoteException, SQLException {
		return dm.getStringFunctions();
	}

	public String getSystemFunctions() throws RemoteException, SQLException {
		return dm.getSystemFunctions();
	}

	public String getTimeDateFunctions() throws RemoteException, SQLException {
		return dm.getTimeDateFunctions();
	}

	public String getSearchStringEscape() throws RemoteException, SQLException {
		return dm.getSearchStringEscape();
	}

	public String getExtraNameCharacters() throws RemoteException, SQLException {
		return dm.getExtraNameCharacters();
	}

	//--------------------------------------------------------------------
	// Functions describing which features are supported.

	public boolean supportsAlterTableWithAddColumn() throws RemoteException, SQLException {
		return dm.supportsAlterTableWithAddColumn();
	}

	public boolean supportsAlterTableWithDropColumn() throws RemoteException, SQLException {
		return dm.supportsAlterTableWithDropColumn();
	}

	public boolean supportsColumnAliasing() throws RemoteException, SQLException {
		return dm.supportsColumnAliasing();
	}

	public boolean nullPlusNonNullIsNull() throws RemoteException, SQLException {
		return dm.nullPlusNonNullIsNull();
	}

	public boolean supportsConvert() throws RemoteException, SQLException {
		return dm.supportsConvert();
	}

	public boolean supportsConvert(int fromType, int toType) throws RemoteException, SQLException {
		return dm.supportsConvert(fromType, toType);
	}

	public boolean supportsTableCorrelationNames() throws RemoteException, SQLException {
		return dm.supportsTableCorrelationNames();
	}

	public boolean supportsDifferentTableCorrelationNames() throws RemoteException, SQLException {
		return dm.supportsDifferentTableCorrelationNames();
	}

	public boolean supportsExpressionsInOrderBy() throws RemoteException, SQLException {
		return dm.supportsExpressionsInOrderBy();
	}

	public boolean supportsOrderByUnrelated() throws RemoteException, SQLException {
		return dm.supportsOrderByUnrelated();
	}

	public boolean supportsGroupBy() throws RemoteException, SQLException {
		return dm.supportsGroupBy();
	}

	public boolean supportsGroupByUnrelated() throws RemoteException, SQLException {
		return dm.supportsGroupByUnrelated();
	}

	public boolean supportsGroupByBeyondSelect() throws RemoteException, SQLException {
		return dm.supportsGroupByBeyondSelect();
	}

	public boolean supportsLikeEscapeClause() throws RemoteException, SQLException {
		return dm.supportsLikeEscapeClause();
	}

	public boolean supportsMultipleResultSets() throws RemoteException, SQLException {
		return dm.supportsMultipleResultSets();
	}

	public boolean supportsMultipleTransactions() throws RemoteException, SQLException {
		return dm.supportsMultipleTransactions();
	}

	public boolean supportsNonNullableColumns() throws RemoteException, SQLException {
		return dm.supportsNonNullableColumns();
	}

	public boolean supportsMinimumSQLGrammar() throws RemoteException, SQLException {
		return dm.supportsMinimumSQLGrammar();
	}

	public boolean supportsCoreSQLGrammar() throws RemoteException, SQLException {
		return dm.supportsCoreSQLGrammar();
	}

	public boolean supportsExtendedSQLGrammar() throws RemoteException, SQLException {
		return dm.supportsExtendedSQLGrammar();
	}

	public boolean supportsANSI92EntryLevelSQL() throws RemoteException, SQLException {
		return dm.supportsANSI92EntryLevelSQL();
	}

	public boolean supportsANSI92IntermediateSQL() throws RemoteException, SQLException {
		return dm.supportsANSI92IntermediateSQL();
	}

	public boolean supportsANSI92FullSQL() throws RemoteException, SQLException {
		return dm.supportsANSI92FullSQL();
	}

	public boolean supportsIntegrityEnhancementFacility() throws RemoteException, SQLException {
		return dm.supportsIntegrityEnhancementFacility();
	}

	public boolean supportsOuterJoins() throws RemoteException, SQLException {
		return dm.supportsOuterJoins();
	}

	public boolean supportsFullOuterJoins() throws RemoteException, SQLException {
		return dm.supportsFullOuterJoins();
	}

	public boolean supportsLimitedOuterJoins() throws RemoteException, SQLException {
		return dm.supportsLimitedOuterJoins();
	}

	public String getSchemaTerm() throws RemoteException, SQLException {
		return dm.getSchemaTerm();
	}

	public String getProcedureTerm() throws RemoteException, SQLException {
		return dm.getProcedureTerm();
	}

	public String getCatalogTerm() throws RemoteException, SQLException {
		return dm.getCatalogTerm();
	}

	public boolean isCatalogAtStart() throws RemoteException, SQLException {
		return dm.isCatalogAtStart();
	}

	public String getCatalogSeparator() throws RemoteException, SQLException {
		return dm.getCatalogSeparator();
	}

	public boolean supportsSchemasInDataManipulation() throws RemoteException, SQLException {
		return dm.supportsSchemasInDataManipulation();
	}

	public boolean supportsSchemasInProcedureCalls() throws RemoteException, SQLException {
		return dm.supportsSchemasInProcedureCalls();
	}

	public boolean supportsSchemasInTableDefinitions() throws RemoteException, SQLException {
		return dm.supportsSchemasInTableDefinitions();
	}

	public boolean supportsSchemasInIndexDefinitions() throws RemoteException, SQLException {
		return dm.supportsSchemasInIndexDefinitions();
	}

	public boolean supportsSchemasInPrivilegeDefinitions() throws RemoteException, SQLException {
		return dm.supportsSchemasInPrivilegeDefinitions();
	}

	public boolean supportsCatalogsInDataManipulation() throws RemoteException, SQLException {
		return dm.supportsCatalogsInDataManipulation();
	}

	public boolean supportsCatalogsInProcedureCalls() throws RemoteException, SQLException {
		return dm.supportsCatalogsInProcedureCalls();
	}

	public boolean supportsCatalogsInTableDefinitions() throws RemoteException, SQLException {
		return dm.supportsCatalogsInTableDefinitions();
	}

	public boolean supportsCatalogsInIndexDefinitions() throws RemoteException, SQLException {
		return dm.supportsCatalogsInIndexDefinitions();
	}

	public boolean supportsCatalogsInPrivilegeDefinitions() throws RemoteException, SQLException {
		return dm.supportsCatalogsInPrivilegeDefinitions();
	}

	public boolean supportsPositionedDelete() throws RemoteException, SQLException {
		return dm.supportsPositionedDelete();
	}

	public boolean supportsPositionedUpdate() throws RemoteException, SQLException {
		return dm.supportsPositionedUpdate();
	}

	public boolean supportsSelectForUpdate() throws RemoteException, SQLException {
		return dm.supportsSelectForUpdate();
	}

	public boolean supportsStoredProcedures() throws RemoteException, SQLException {
		return dm.supportsStoredProcedures();
	}

	public boolean supportsSubqueriesInComparisons() throws RemoteException, SQLException {
		return dm.supportsSubqueriesInComparisons();
	}

	public boolean supportsSubqueriesInExists() throws RemoteException, SQLException {
		return dm.supportsSubqueriesInExists();
	}

	public boolean supportsSubqueriesInIns() throws RemoteException, SQLException {
		return dm.supportsSubqueriesInIns();
	}

	public boolean supportsSubqueriesInQuantifieds() throws RemoteException, SQLException {
		return dm.supportsSubqueriesInQuantifieds();
	}

	public boolean supportsCorrelatedSubqueries() throws RemoteException, SQLException {
		return dm.supportsCorrelatedSubqueries();
	}

	public boolean supportsUnion() throws RemoteException, SQLException {
		return dm.supportsUnion();
	}

	public boolean supportsUnionAll() throws RemoteException, SQLException {
		return dm.supportsUnionAll();
	}

	public boolean supportsOpenCursorsAcrossCommit() throws RemoteException, SQLException {
		return dm.supportsOpenCursorsAcrossCommit();
	}

	public boolean supportsOpenCursorsAcrossRollback() throws RemoteException, SQLException {
		return dm.supportsOpenCursorsAcrossRollback();
	}

	public boolean supportsOpenStatementsAcrossCommit() throws RemoteException, SQLException {
		return dm.supportsOpenStatementsAcrossCommit();
	}

	public boolean supportsOpenStatementsAcrossRollback() throws RemoteException, SQLException {
		return dm.supportsOpenStatementsAcrossRollback();
	}

	//----------------------------------------------------------------------
	// The following group of methods exposes various limitations
	// based on the target database with the current driver.
	// Unless otherwise specified, a result of zero means there is no
	// limit, or the limit is not known.

	public int getMaxBinaryLiteralLength() throws RemoteException, SQLException {
		return dm.getMaxBinaryLiteralLength();
	}

	public int getMaxCharLiteralLength() throws RemoteException, SQLException {
		return dm.getMaxCharLiteralLength();
	}

	public int getMaxColumnNameLength() throws RemoteException, SQLException {
		return dm.getMaxColumnNameLength();
	}

	public int getMaxColumnsInGroupBy() throws RemoteException, SQLException {
		return dm.getMaxColumnsInGroupBy();
	}

	public int getMaxColumnsInIndex() throws RemoteException, SQLException {
		return dm.getMaxColumnsInIndex();
	}

	public int getMaxColumnsInOrderBy() throws RemoteException, SQLException {
		return dm.getMaxColumnsInOrderBy();
	}

	public int getMaxColumnsInSelect() throws RemoteException, SQLException {
		return dm.getMaxColumnsInSelect();
	}

	public int getMaxColumnsInTable() throws RemoteException, SQLException {
		return dm.getMaxColumnsInTable();
	}

	public int getMaxConnections() throws RemoteException, SQLException {
		return dm.getMaxConnections();
	}

	public int getMaxCursorNameLength() throws RemoteException, SQLException {
		return dm.getMaxCursorNameLength();
	}

	public int getMaxIndexLength() throws RemoteException, SQLException {
		return dm.getMaxIndexLength();
	}

	public int getMaxSchemaNameLength() throws RemoteException, SQLException {
		return dm.getMaxSchemaNameLength();
	}

	public int getMaxProcedureNameLength() throws RemoteException, SQLException {
		return dm.getMaxProcedureNameLength();
	}

	public int getMaxCatalogNameLength() throws RemoteException, SQLException {
		return dm.getMaxCatalogNameLength();
	}

	public int getMaxRowSize() throws RemoteException, SQLException {
		return dm.getMaxRowSize();
	}

	public boolean doesMaxRowSizeIncludeBlobs() throws RemoteException, SQLException {
		return dm.doesMaxRowSizeIncludeBlobs();
	}

	public int getMaxStatementLength() throws RemoteException, SQLException {
		return dm.getMaxStatementLength();
	}

	public int getMaxStatements() throws RemoteException, SQLException {
		return dm.getMaxStatements();
	}

	public int getMaxTableNameLength() throws RemoteException, SQLException {
		return dm.getMaxTableNameLength();
	}

	public int getMaxTablesInSelect() throws RemoteException, SQLException {
		return dm.getMaxTablesInSelect();
	}

	public int getMaxUserNameLength() throws RemoteException, SQLException {
		return dm.getMaxUserNameLength();
	}

	//----------------------------------------------------------------------

	public int getDefaultTransactionIsolation() throws RemoteException, SQLException {
		return dm.getDefaultTransactionIsolation();
	}

	public boolean supportsTransactions() throws RemoteException, SQLException {
		return dm.supportsTransactions();
	}

	public boolean supportsTransactionIsolationLevel(int level) throws RemoteException, SQLException {
		return dm.supportsTransactionIsolationLevel(level);
	}

	public boolean supportsDataDefinitionAndDataManipulationTransactions() throws RemoteException, SQLException {
		return dm.supportsDataDefinitionAndDataManipulationTransactions();
	}

	public boolean supportsDataManipulationTransactionsOnly() throws RemoteException, SQLException {
		return dm.supportsDataManipulationTransactionsOnly();
	}

	public boolean dataDefinitionCausesTransactionCommit() throws RemoteException, SQLException {
		return dm.dataDefinitionCausesTransactionCommit();
	}

	public boolean dataDefinitionIgnoredInTransactions() throws RemoteException, SQLException {
		return dm.dataDefinitionIgnoredInTransactions();
	}

	public RemoteResultSet getProcedures(String catalog, String schemaPattern, String procedureNamePattern) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getProcedures(catalog, schemaPattern, procedureNamePattern));
	}

	public RemoteResultSet getProcedureColumns(String catalog, String schemaPattern, String procedureNamePattern, String columnNamePattern) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getProcedureColumns(catalog, schemaPattern, procedureNamePattern, columnNamePattern));
	}

	public RemoteResultSet getTables(String catalog, String schemaPattern, String tableNamePattern, String types[]) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getTables(catalog, schemaPattern, tableNamePattern, types));
	}

	public RemoteResultSet getSchemas() throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getSchemas());
	}

	public RemoteResultSet getCatalogs() throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getCatalogs());
	}

	public RemoteResultSet getTableTypes() throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getTableTypes());
	}

	public RemoteResultSet getColumns(String catalog, String schemaPattern, String tableNamePattern, String columnNamePattern) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getColumns(catalog, schemaPattern, tableNamePattern, columnNamePattern));
	}

	public RemoteResultSet getColumnPrivileges(String catalog, String schema, String table, String columnNamePattern) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getColumnPrivileges(catalog, schema, table, columnNamePattern));
	}

	public RemoteResultSet getTablePrivileges(String catalog, String schemaPattern, String tableNamePattern) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getTablePrivileges(catalog, schemaPattern, tableNamePattern));
	}

	public RemoteResultSet getBestRowIdentifier(String catalog, String schema, String table, int scope, boolean nullable) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getBestRowIdentifier(catalog, schema, table, scope, nullable));
	}

	public RemoteResultSet getVersionColumns(String catalog, String schema, String table) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getVersionColumns(catalog, schema, table));
	}

	public RemoteResultSet getPrimaryKeys(String catalog, String schema, String table) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getPrimaryKeys(catalog, schema, table));
	}

	public RemoteResultSet getImportedKeys(String catalog, String schema, String table) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getImportedKeys(catalog, schema, table));
	}

	public RemoteResultSet getExportedKeys(String catalog, String schema, String table) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getExportedKeys(catalog, schema, table));
	}

	public RemoteResultSet getCrossReference(String primaryCatalog, String primarySchema, String primaryTable, String foreignCatalog, String foreignSchema, String foreignTable) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getCrossReference(primaryCatalog, primarySchema, primaryTable, foreignCatalog, foreignSchema, foreignTable));
	}

	public RemoteResultSet getTypeInfo() throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getTypeInfo());
	}

	public RemoteResultSet getIndexInfo(String catalog, String schema, String table, boolean unique, boolean approximate) throws RemoteException, SQLException {
		return new RemoteResultSetImpl(dm.getIndexInfo(catalog, schema, table, unique, approximate));
	}
}

class RemoteResultSetMetaDataImpl extends UnicastRemoteObject implements RemoteResultSetMetaData {
	private ResultSetMetaData rsmd;

	public RemoteResultSetMetaDataImpl(ResultSetMetaData rsmd) throws RemoteException {
		this.rsmd = rsmd;
	}

	//----------------------------------------------------------------------

	public int getColumnCount() throws RemoteException, SQLException {
		return rsmd.getColumnCount();
	}

	public boolean isAutoIncrement(int column) throws RemoteException, SQLException {
		return rsmd.isAutoIncrement(column);
	}

	public boolean isCaseSensitive(int column) throws RemoteException, SQLException {
		return rsmd.isCaseSensitive(column);
	}

	public boolean isSearchable(int column) throws RemoteException, SQLException {
		return rsmd.isSearchable(column);
	}

	public boolean isCurrency(int column) throws RemoteException, SQLException {
		return rsmd.isCurrency(column);
	}

	public int isNullable(int column) throws RemoteException, SQLException {
		return rsmd.isNullable(column);
	}

	public boolean isSigned(int column) throws RemoteException, SQLException {
		return rsmd.isSigned(column);
	}

	public int getColumnDisplaySize(int column) throws RemoteException, SQLException {
		return rsmd.getColumnDisplaySize(column);
	}

	public String getColumnLabel(int column) throws RemoteException, SQLException {
		return rsmd.getColumnLabel(column);
	}

	public String getColumnName(int column) throws RemoteException, SQLException {
		return rsmd.getColumnName(column);
	}

	public String getSchemaName(int column) throws RemoteException, SQLException {
		return rsmd.getSchemaName(column);
	}

	public int getPrecision(int column) throws RemoteException, SQLException {
		return rsmd.getPrecision(column);
	}

	public int getScale(int column) throws RemoteException, SQLException {
		return rsmd.getScale(column);
	}

	public String getTableName(int column) throws RemoteException, SQLException {
		return rsmd.getTableName(column);
	}

	public String getCatalogName(int column) throws RemoteException, SQLException {
		return rsmd.getCatalogName(column);
	}

	public int getColumnType(int column) throws RemoteException, SQLException {
		return rsmd.getColumnType(column);
	}

	public String getColumnTypeName(int column) throws RemoteException, SQLException {
		return rsmd.getColumnTypeName(column);
	}

	public boolean isReadOnly(int column) throws RemoteException, SQLException {
		return rsmd.isReadOnly(column);
	}

	public boolean isWritable(int column) throws RemoteException, SQLException {
		return rsmd.isWritable(column);
	}

	public boolean isDefinitelyWritable(int column) throws RemoteException, SQLException {
		return rsmd.isDefinitelyWritable(column);
	}
}

class RemoteInputStreamImpl extends UnicastRemoteObject implements RemoteInputStream {
	private InputStream in;

	public RemoteInputStreamImpl(InputStream in) throws RemoteException {
		this.in = in;
	}

	//----------------------------------------------------------------------

	public int read() throws RemoteException, IOException {
		return in.read();
	}

	public int read(byte b[]) throws RemoteException, IOException {
		return in.read(b);
	}

	public int read(byte b[], int off, int len) throws RemoteException, IOException {
		return in.read(b, off, len);
	}

	public long skip(long n) throws RemoteException, IOException {
		return in.skip(n);
	}

	public int available() throws RemoteException, IOException {
		return in.available();
	}

	public void close() throws RemoteException, IOException {
		in.close();
	}
}
