//
//	RemoteResultSet.java
//
//	version 1.0  98/12/12
//	author  Mitsumasa IWATSUKI
//

package org.iwatsuki.rjj;

import java.math.BigDecimal;
import java.rmi.*;
import java.sql.*;

interface RemoteResultSet extends Remote {
	boolean next() throws RemoteException, SQLException;
	void close() throws RemoteException, SQLException;
	boolean wasNull() throws RemoteException, SQLException;

	//======================================================================
	// Methods for accessing results by column index
	//======================================================================

	String getString(int columnIndex) throws RemoteException, SQLException;
	boolean getBoolean(int columnIndex) throws RemoteException, SQLException;
	byte getByte(int columnIndex) throws RemoteException, SQLException;
	short getShort(int columnIndex) throws RemoteException, SQLException;
	int getInt(int columnIndex) throws RemoteException, SQLException;
	long getLong(int columnIndex) throws RemoteException, SQLException;
	float getFloat(int columnIndex) throws RemoteException, SQLException;
	double getDouble(int columnIndex) throws RemoteException, SQLException;
	BigDecimal getBigDecimal(int columnIndex, int scale) throws RemoteException, SQLException;
	byte[] getBytes(int columnIndex) throws RemoteException, SQLException;
	java.sql.Date getDate(int columnIndex) throws RemoteException, SQLException;
	java.sql.Time getTime(int columnIndex) throws RemoteException, SQLException;
	java.sql.Timestamp getTimestamp(int columnIndex) throws RemoteException, SQLException;
	RemoteInputStream getAsciiStream(int columnIndex) throws RemoteException, SQLException;
	RemoteInputStream getUnicodeStream(int columnIndex) throws RemoteException, SQLException;
	RemoteInputStream getBinaryStream(int columnIndex) throws RemoteException, SQLException;

	//======================================================================
	// Methods for accessing results by column name
	//======================================================================

	String getString(String columnName) throws RemoteException, SQLException;
	boolean getBoolean(String columnName) throws RemoteException, SQLException;
	byte getByte(String columnName) throws RemoteException, SQLException;
	short getShort(String columnName) throws RemoteException, SQLException;
	int getInt(String columnName) throws RemoteException, SQLException;
	long getLong(String columnName) throws RemoteException, SQLException;
	float getFloat(String columnName) throws RemoteException, SQLException;
	double getDouble(String columnName) throws RemoteException, SQLException;
	BigDecimal getBigDecimal(String columnName, int scale) throws RemoteException, SQLException;
	byte[] getBytes(String columnName) throws RemoteException, SQLException;
	java.sql.Date getDate(String columnName) throws RemoteException, SQLException;
	java.sql.Time getTime(String columnName) throws RemoteException, SQLException;
	java.sql.Timestamp getTimestamp(String columnName) throws RemoteException, SQLException;
	RemoteInputStream getAsciiStream(String columnName) throws RemoteException, SQLException;
	RemoteInputStream getUnicodeStream(String columnName) throws RemoteException, SQLException;
	RemoteInputStream getBinaryStream(String columnName) throws RemoteException, SQLException;

	//=====================================================================
	// Advanced features:
	//=====================================================================

	SQLWarning getWarnings() throws RemoteException, SQLException;
	void clearWarnings() throws RemoteException, SQLException;
	String getCursorName() throws RemoteException, SQLException;
	RemoteResultSetMetaData getMetaData() throws RemoteException, SQLException;
	Object getObject(int columnIndex) throws RemoteException, SQLException;
	Object getObject(String columnName) throws RemoteException, SQLException;

	//----------------------------------------------------------------

	int findColumn(String columnName) throws RemoteException, SQLException;
}
